@@ -15,8 +15,12 @@ clean () {
 	rm -rf $WORKING_DIR
 }
 
+git clone git://sentinella.git.sourceforge.net/gitroot/sentinella/sentinella -b v${DOTTED_VERSION%.*} sentinella-$DOTTED_VERSION
+cd sentinella-$DOTTED_VERSION
+git checkout v$DOTTED_VERSION
+cd ..
+
 mkdir -p $WORKING_DIR
-cvs -d:pserver:anonymous@sentinella.cvs.sourceforge.net:/cvsroot/sentinella export -r sentinella-$SLASHED_VERSION -d sentinella-$DOTTED_VERSION Sentinella
 mv sentinella-$DOTTED_VERSION $WORKING_DIR
 
 read -p "Have you updated the version in main.cpp? (y/n) " yn
@@ -36,11 +40,12 @@ mkdir $WORKING_DIR/sentinella-$DOTTED_VERSION/build
 cd $WORKING_DIR/sentinella-$DOTTED_VERSION/build/
 cmake .. -DCMAKE_BUILD_TYPE=debug && make
 if	[ $? != 0 ]; then
-	echo "ERROR: Can't build the CVS code."
+	echo "ERROR: Can't build the git code."
 	clean
 	exit 4
 fi
 rm -rf $WORKING_DIR/sentinella-$DOTTED_VERSION/build/*
+rm -rf $WORKING_DIR/sentinella-$DOTTED_VERSION/.git*
 
 chmod -x $WORKING_DIR/sentinella-$DOTTED_VERSION/src/Actions/*
 chmod -x $WORKING_DIR/sentinella-$DOTTED_VERSION/src/Conditions/*
