@@ -1,7 +1,7 @@
 /*
  * Sentinella
- * http://sourceforge.net/projects/sentinella/
- * Copyright (c) 2009, 2010 Carlos Olmedo Escobar <carlos.olmedo.e@gmail.com>
+ * http://sentinella.sourceforge.net/
+ * Copyright (c) 2009-2011 Carlos Olmedo Escobar <carlos.olmedo.e@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +15,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * Foun dation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "MainWindow.h"
 #include <errno.h>
-#include <libsysactivity/libsysactivity.h>
 #include <QFile>
 #include <QDBusInterface>
 #include <QDBusReply>
@@ -53,56 +52,44 @@
 #include "Actions/Shutdown.h"
 #include "Actions/Sleep.h"
 
-// Ids of the condition radiobuttons
-#define RADIOBUTTONCPU 0
-#define RADIOBUTTONMEMORY 1
-#define RADIOBUTTONNETWORK 2
-#define RADIOBUTTONDATETIME 3
-#define RADIOBUTTONPROCESSDIES 4
-
-// Ids of the action radiobuttons (shutdown and reboot are the same because they use the same QStackedWiget page)
-#define RADIOBUTTONSHUTDOWN 0
-#define RADIOBUTTONREBOOT 0
-#define RADIOBUTTONSLEEP 1
-#define RADIOBUTTONEXECUTECOMMAND 2
-#define RADIOBUTTONPLAYALARM 3
-#define RADIOBUTTONKILLPROCESS 4
-
-MainWindow::MainWindow(const KAboutData& aboutData, KCmdLineArgs* cmdLineArgs,
-		const short cmdSelectedCondition, const short cmdSelectedAction) :
-	KMainWindow(NULL, Qt::WindowContextHelpButtonHint|Qt::WindowCloseButtonHint), aboutData(aboutData), cmdLineArgs(cmdLineArgs) {
+MainWindow::MainWindow(KCmdLineArgs* cmdLineArgs, const ConditionId cmdChosenCondition,
+		const ActionId cmdChosenAction) :
+	KMainWindow(NULL, Qt::WindowContextHelpButtonHint | Qt::WindowCloseButtonHint), cmdLineArgs(
+			cmdLineArgs) {
 	condition = NULL;
 	action = NULL;
 	supportedShutdown = false;
 	pidProcessDies = pidKillProcess = 0;
 	availableSounds = NULL;
+	processAPI = NULL;
+	networkInterfaceAPI = NULL;
 
-	checkShutdownSleepStates();
-	config = KGlobal::config();
 	iconLoader = KIconLoader::global();
+	config = KGlobal::config();
+	checkShutdownSleepStates();
 
 	gui.setupUi(this);
 	gui.pushButtonStartStop->setIcon(QIcon(iconLoader->loadIcon("arrow-right", KIconLoader::Small)));
 
 	trayIcon.setAssociatedWidget(this);
 	trayIcon.setCategory(KStatusNotifierItem::ApplicationStatus);
-	trayIcon.setStatus(KStatusNotifierItem::Active);
+	trayIcon.setStatus(KStatusNotifierItem::Passive);
 	trayIcon.contextMenu()->addAction(KStandardAction::aboutApp(this, SLOT(showAboutApp()),
 			trayIcon.contextMenu()));
 	trayIcon.setIconByName("sentinella");
 
-	conditionsGroup.addButton(gui.radioButtonCPU, RADIOBUTTONCPU);
-	conditionsGroup.addButton(gui.radioButtonMemory, RADIOBUTTONMEMORY);
-	conditionsGroup.addButton(gui.radioButtonNetwork, RADIOBUTTONNETWORK);
-	conditionsGroup.addButton(gui.radioButtonDateTime, RADIOBUTTONDATETIME);
-	conditionsGroup.addButton(gui.radioButtonProcessDies, RADIOBUTTONPROCESSDIES);
+	conditionsGroup.addButton(gui.radioButtonCPU, CPU_ID);
+	conditionsGroup.addButton(gui.radioButtonMemory, MEMORY);
+	conditionsGroup.addButton(gui.radioButtonNetwork, NETWORK);
+	conditionsGroup.addButton(gui.radioButtonDateTime, DATETIME);
+	conditionsGroup.addButton(gui.radioButtonProcessDies, PROCESSDIES);
 	connect(gui.radioButtonNetwork, SIGNAL(toggled(bool)), this, SLOT(radioButtonNetworkToggled(bool)));
 	connect(&conditionsGroup, SIGNAL(buttonClicked(int)), gui.stackedWidgetConditions, SLOT(setCurrentIndex(int)));
 	connect(gui.pushButtonProcessDies, SIGNAL(clicked(bool)), this, SLOT(programDiesClicked()));
 
 	if (supportedShutdown) {
-		actionsGroup.addButton(gui.radioButtonShutdown, RADIOBUTTONSHUTDOWN);
-		actionsGroup.addButton(gui.radioButtonReboot, RADIOBUTTONREBOOT);
+		actionsGroup.addButton(gui.radioButtonShutdown, SHUTDOWN);
+		actionsGroup.addButton(gui.radioButtonReboot, REBOOT);
 	} else {
 		gui.radioButtonShutdown->setEnabled(false);
 		gui.radioButtonReboot->setEnabled(false);
@@ -110,26 +97,31 @@ MainWindow::MainWindow(const KAboutData& aboutData, KCmdLineArgs* cmdLineArgs,
 	if (supportedSleepStates.empty())
 		gui.radioButtonSleep->setEnabled(false);
 	else
-		actionsGroup.addButton(gui.radioButtonSleep, RADIOBUTTONSLEEP);
-	actionsGroup.addButton(gui.radioButtonExecuteCommand, RADIOBUTTONEXECUTECOMMAND);
-	actionsGroup.addButton(gui.radioButtonPlayAlarm, RADIOBUTTONPLAYALARM);
-	actionsGroup.addButton(gui.radioButtonKillProcess, RADIOBUTTONKILLPROCESS);
+		actionsGroup.addButton(gui.radioButtonSleep, SLEEP);
+	actionsGroup.addButton(gui.radioButtonExecuteCommand, EXECUTECOMMAND);
+	actionsGroup.addButton(gui.radioButtonPlayAlarm, PLAYALARM);
+	actionsGroup.addButton(gui.radioButtonKillProcess, KILLPROCESS);
 	connect(gui.radioButtonPlayAlarm, SIGNAL(toggled(bool)), this, SLOT(radioButtonPlayAlarmToggled(bool)));
-	connect(&actionsGroup, SIGNAL(buttonClicked(int)), gui.stackedWidgetActions, SLOT(setCurrentIndex(int)));
+	connect(&actionsGroup, SIGNAL(buttonClicked(int)), this, SLOT(actionButtonClicked(int)));
 	connect(gui.pushButtonKillProcess, SIGNAL(clicked(bool)), this, SLOT(killProcessClicked()));
 
 	connect(gui.pushButtonStartStop, SIGNAL(clicked(bool)), this, SLOT(startStopClicked()));
 
-	initConditionWidgets(cmdSelectedCondition);
-	initActionWidgets(cmdSelectedAction);
-	if (cmdLineArgs->isSet("start"))
-		startClicked();
+	initConditionWidgets(cmdChosenCondition);
+	initActionWidgets(cmdChosenAction);
+	if (cmdLineArgs != NULL) {
+		if (cmdLineArgs->isSet("start"))
+			startCondition();
 
-	cmdLineArgs->clear(); // the given cmd args are not needed anymore
+		cmdLineArgs->clear(); // the given cmd args are not needed anymore
+	}
 }
 
 MainWindow::~MainWindow() {
 	delete availableSounds;
+	delete processAPI;
+	delete networkInterfaceAPI;
+	delete condition;
 }
 
 void MainWindow::checkShutdownSleepStates() {
@@ -142,14 +134,14 @@ void MainWindow::checkShutdownSleepStates() {
 		isKDE = true;
 		delete interface;
 
-		interface = new QDBusInterface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil");
-		QDBusReply<QVariantMap> mapReply = interface->call("getSupportedSuspendMethods");
-		if (mapReply.isValid()) {
-			QList<QVariant> values = mapReply.value().values();
-			for (int i = 0; i < values.size(); i++)
-				supportedSleepStates.insert(
-						(Solid::PowerManagement::SleepState) values.at(i).toInt());
-		}
+		interface = new QDBusInterface("org.kde.Solid.PowerManagement",
+				"/org/freedesktop/PowerManagement", "org.freedesktop.PowerManagement");
+		boolReply = interface->call("CanSuspend");
+		if (boolReply.isValid() && boolReply)
+			supportedSleepStates.insert(Solid::PowerManagement::SuspendState);
+		boolReply = interface->call("CanHibernate");
+		if (boolReply.isValid() && boolReply)
+			supportedSleepStates.insert(Solid::PowerManagement::HibernateState);
 	} else {
 		//see http://upower.freedesktop.org/docs/UPower.html
 		delete interface;
@@ -176,118 +168,98 @@ void MainWindow::checkShutdownSleepStates() {
 	delete interface;
 }
 
-void MainWindow::initConditionWidgets(const short cmdSelectedCondition) {
-	int aux;
-	if (cmdSelectedCondition != -1)
-		aux = cmdSelectedCondition;
+void MainWindow::initConditionWidgets(const ConditionId cmdChosenCondition) {
+	ConditionId chosen;
+	if (cmdChosenCondition != NO_CONDITION)
+		chosen = cmdChosenCondition;
 	else {
 		KConfigGroup group = config->group("Conditions");
-		aux = group.readEntry("LastExecuted", 0);
-		if (aux < 0 || aux > 4)
-			aux = 0;
+		chosen = (ConditionId) group.readEntry("LastExecuted", 0);
+		if (chosen < CPU_ID || chosen > PROCESSDIES)
+			chosen = CPU_ID;
 	}
 
-	conditionsGroup.button(aux)->setChecked(true);
-	gui.stackedWidgetConditions->setCurrentIndex(aux);
+	conditionsGroup.button(chosen)->setChecked(true);
+	gui.stackedWidgetConditions->setCurrentIndex(chosen);
 
-	if (cmdLineArgs->isSet("higher"))
+	int aux;
+	if (cmdLineArgs != NULL && cmdLineArgs->isSet("higher"))
 		aux = 0;
-	else if (cmdLineArgs->isSet("lower"))
+	else if (cmdLineArgs != NULL && cmdLineArgs->isSet("lower"))
 		aux = 1;
 	else
 		aux = -1;
 
 	KConfigGroup groupCPU = config->group("ConditionCPU");
 	gui.comboBoxCPU->setCurrentIndex(aux != -1 ? aux : groupCPU.readEntry("Higher", 1));
-	gui.spinBoxCPU->setValue(cmdSelectedCondition == 0 ? cmdLineArgs->getOption("cpu").toInt()
+	gui.spinBoxCPU->setValue(cmdChosenCondition == CPU_ID ? cmdLineArgs->getOption("cpu").toInt()
 			: groupCPU.readEntry("Percentage", 10));
-	gui.timeEditCPU->setTime(QTime::fromString(cmdSelectedCondition == 0 && cmdLineArgs->isSet(
+	gui.timeEditCPU->setTime(QTime::fromString(cmdChosenCondition == CPU_ID && cmdLineArgs->isSet(
 			"during") ? cmdLineArgs->getOption("during") : groupCPU.readEntry("Time", "01:00"),
 			"mm:ss"));
 
 	KConfigGroup groupMemory = config->group("ConditionMemory");
 	gui.comboBoxMemory->setCurrentIndex(aux != -1 ? aux : groupMemory.readEntry("Higher", 1));
 	gui.spinBoxMemory->setValue(
-			cmdSelectedCondition == 1 ? cmdLineArgs->getOption("memory").toInt()
+			cmdChosenCondition == MEMORY ? cmdLineArgs->getOption("memory").toInt()
 					: groupMemory.readEntry("Amount", 512));
-	gui.timeEditMemory->setTime(QTime::fromString(cmdSelectedCondition == 1 && cmdLineArgs->isSet(
-			"during") ? cmdLineArgs->getOption("during") : groupMemory.readEntry("Time", "01:00"),
-			"mm:ss"));
+	gui.timeEditMemory->setTime(QTime::fromString(cmdChosenCondition == MEMORY
+			&& cmdLineArgs->isSet("during") ? cmdLineArgs->getOption("during")
+			: groupMemory.readEntry("Time", "01:00"), "mm:ss"));
 
 	KConfigGroup groupNetwork = config->group("ConditionNetwork");
 	gui.comboBoxNetworkDirection->setCurrentIndex(groupNetwork.readEntry("Direction", 0));
 	gui.comboBoxNetworkHigher->setCurrentIndex(aux != -1 ? aux
 			: groupNetwork.readEntry("Higher", 0));
 	gui.spinBoxNetwork->setValue(
-			cmdSelectedCondition == 2 ? cmdLineArgs->getOption("network").toInt()
+			cmdChosenCondition == NETWORK ? cmdLineArgs->getOption("network").toInt()
 					: groupNetwork.readEntry("Amount", 10));
-	gui.timeEditNetwork->setTime(QTime::fromString(cmdSelectedCondition == 2 && cmdLineArgs->isSet(
-			"during") ? cmdLineArgs->getOption("during") : groupNetwork.readEntry("Time", "01:00"),
-			"mm:ss"));
+	gui.timeEditNetwork->setTime(QTime::fromString(cmdChosenCondition == NETWORK
+			&& cmdLineArgs->isSet("during") ? cmdLineArgs->getOption("during")
+			: groupNetwork.readEntry("Time", "01:00"), "mm:ss"));
 
 	QDateTime dateTime;
-	if (cmdSelectedCondition == 3)
+	if (cmdChosenCondition == DATETIME)
 		dateTime = QDateTime::fromString(cmdLineArgs->getOption("date-time"), Qt::ISODate);
-	if (cmdSelectedCondition != 3 || !dateTime.isValid()) {
+	if (cmdChosenCondition != DATETIME || !dateTime.isValid()) {
 		KConfigGroup groupDateTime = config->group("ConditionDateTime");
-		dateTime = groupDateTime.readEntry("DateTime", QDateTime::currentDateTime());
+		dateTime = QDateTime::currentDateTime().addSecs(
+				groupDateTime.readEntry("Interval", "1800").toInt());
 	}
 	QTime readTime(dateTime.time().hour(), dateTime.time().minute(), 0); // Setting 0 seconds
 	dateTime.setTime(readTime);
 	gui.kdatetimewidgetDateTime->setDateTime(dateTime);
 
-	if (cmdSelectedCondition == 4) {
-#ifdef SA_OPEN_PROCESS
-		if (sa_open_process() != 0) {
-#ifdef SA_CLOSE_PROCESS
-			sa_close_process();
-#endif
-			return;
-		}
-#endif
+	if (cmdChosenCondition == PROCESSDIES) {
 		struct sa_process procInfo;
-		if (sa_get_process(cmdLineArgs->getOption("program-dies").toInt(), &procInfo) == 0) {
+		if ((procInfo.pid = cmdLineArgs->getOption("program-dies").toInt()) == 0)
+			return;
+		if (getProcess(&procInfo)) {
 			pidProcessDies = procInfo.pid;
 			gui.labelProcessDies->setText(QString::fromAscii(procInfo.filename));
 		}
-#ifdef SA_CLOSE_PROCESS
-		sa_close_process();
-#endif
 	}
 }
 
-void MainWindow::initActionWidgets(const short cmdSelectedAction) {
-	int chosen;
-	if (cmdSelectedAction != -1)
-		chosen = cmdSelectedAction;
+void MainWindow::initActionWidgets(const ActionId cmdChosenAction) {
+	ActionId chosen;
+	if (cmdChosenAction != NO_ACTION)
+		chosen = cmdChosenAction;
 	else {
 		KConfigGroup group = config->group("Actions");
-		chosen = group.readEntry("LastExecuted", 0);
-		if (chosen < 0 || chosen > 5)
-			chosen = 0;
+		chosen = (ActionId) group.readEntry("LastExecuted", 0);
+		if (chosen < SHUTDOWN || chosen > KILLPROCESS)
+			chosen = SHUTDOWN;
 	}
 
-	if (((chosen == 0 || chosen == 1) && !supportedShutdown) || (chosen == 2
-			&& supportedSleepStates.empty()))
-		chosen = 3;
-	switch (chosen) {
-	case 0:
-		gui.radioButtonShutdown->setChecked(true);
-		break;
-	case 1:
-		gui.radioButtonReboot->setChecked(true);
-		break;
-	default:
-		actionsGroup.button(chosen - 1)->setChecked(true);
-	}
-	gui.stackedWidgetActions->setCurrentIndex(chosen - 1);
+	actionsGroup.button(chosen)->click();
 
 	if (!isKDE) {
 		gui.checkBoxShutdownForce->setChecked(false);
 		gui.checkBoxShutdownForce->setEnabled(false);
-	} else if (cmdSelectedAction == 0 || cmdSelectedAction == 1) {
+	} else if (cmdChosenAction == SHUTDOWN || cmdChosenAction == REBOOT) {
 		gui.checkBoxShutdownForce->setChecked(cmdLineArgs->getOption(
-				cmdSelectedAction == 0 ? "shutdown" : "reboot") == "yes");
+				cmdChosenAction == SHUTDOWN ? "shutdown" : "reboot") == "yes");
 	} else {
 		KConfigGroup groupShutdown = config->group("ActionShutdown");
 		gui.checkBoxShutdownForce->setChecked(groupShutdown.readEntry("Force", true));
@@ -302,7 +274,7 @@ void MainWindow::initActionWidgets(const short cmdSelectedAction) {
 				Solid::PowerManagement::HibernateState));
 
 		Solid::PowerManagement::SleepState state;
-		if (cmdSelectedAction == 2) { // cmd arg
+		if (cmdChosenAction == SLEEP) { // cmd arg
 			QString cmdSleepArg = cmdLineArgs->getOption("sleep");
 			if (cmdSleepArg == "Standby")
 				state = Solid::PowerManagement::StandbyState;
@@ -333,62 +305,63 @@ void MainWindow::initActionWidgets(const short cmdSelectedAction) {
 		}
 	}
 
-	if (cmdSelectedAction == 5) {
-#ifdef SA_OPEN_PROCESS
-		if (sa_open_process() != 0) {
-#ifdef SA_CLOSE_PROCESS
-			sa_close_process();
-#endif
-			return;
-		}
-#endif
+	if (cmdChosenAction == EXECUTECOMMAND) {
+		QTextStream textStream(stdin);
+		gui.texteditExecuteCommand->setPlainText(textStream.readLine());
+	} else if (chosen == EXECUTECOMMAND) {
+		KConfigGroup groupExecuteCommand = config->group("ActionExecuteCommand");
+		gui.texteditExecuteCommand->setPlainText(groupExecuteCommand.readEntry("Command"));
+	} else if (cmdChosenAction == KILLPROCESS) {
 		struct sa_process procInfo;
-		if (sa_get_process(cmdLineArgs->getOption("terminate-program").toInt(), &procInfo) == 0) {
+		if ((procInfo.pid = cmdLineArgs->getOption("terminate-program").toInt()) == 0)
+			return;
+		if (getProcess(&procInfo)) {
 			pidKillProcess = procInfo.pid;
 			gui.labelKillProcess->setText(QString::fromAscii(procInfo.filename));
 		}
-#ifdef SA_CLOSE_PROCESS
-		sa_close_process();
-#endif
 	}
-
-	KConfigGroup groupExecuteCommand = config->group("ActionExecuteCommand");
-	gui.texteditExecuteCommand->setPlainText(groupExecuteCommand.readEntry("Command"));
 }
 
 void MainWindow::radioButtonNetworkToggled(bool checked) {
 	if (!checked)
 		return;
 
-	// Populating the combobox
-	gui.comboBoxNetworkInterface->clear();
-	QList<QNetworkInterface> interfaces = QNetworkInterface::allInterfaces();
-	int i;
-	for (i = 0; i < interfaces.size(); i++) {
-		const QNetworkInterface interface = interfaces[i];
-		if (interface.flags() & QNetworkInterface::IsUp)
-			gui.comboBoxNetworkInterface->addItem(interface.name());
-	}
-	// Loading config for the combobox (just the first time)
-	static bool doneOnce = false;
-	if (!doneOnce) {
-		QString name;
-		if (cmdLineArgs->isSet("interface"))
-			name = cmdLineArgs->getOption("interface");
-		else {
-			KConfigGroup groupNetwork = config->group("ConditionNetwork");
-#ifdef Q_OS_FREEBSD
-			name = groupNetwork.readEntry("Interface", "em0");
-#else
-			name = groupNetwork.readEntry("Interface", "eth0");
-#endif
-		}
-		i = gui.comboBoxNetworkInterface->findText(name);
-		if (i != -1)
-			gui.comboBoxNetworkInterface->setCurrentIndex(i);
+	disconnect(gui.radioButtonNetwork, SIGNAL(toggled(bool)), this, SLOT(radioButtonNetworkToggled(bool)));
 
-		doneOnce = true;
+	// Populating the combobox
+	QList < QNetworkInterface > interfaces = QNetworkInterface::allInterfaces();
+	QList<QNetworkInterface>::ConstIterator end = interfaces.constEnd();
+	for (QList<QNetworkInterface>::ConstIterator itr = interfaces.constBegin(); itr != end; ++itr) {
+		if (itr->flags() & QNetworkInterface::IsUp)
+			gui.comboBoxNetworkInterface->addItem(itr->name());
 	}
+
+	// Trying to find out which network interface i should choose by default
+	QString interfaceName;
+	KConfigGroup groupNetwork = config->group("ConditionNetwork");
+	if (cmdLineArgs != NULL && cmdLineArgs->isSet("interface")) // First the cmd arguments
+		interfaceName = cmdLineArgs->getOption("interface");
+	else if (groupNetwork.hasKey("Interface")) // Trying with the config
+		interfaceName = groupNetwork.readEntry("Interface");
+	else { // Choosing the interface with more traffic
+		networkInterfaceAPI = new NetworkInterface();
+		struct sa_net_interface* interfaces;
+		uint16_t written;
+		interfaces = networkInterfaceAPI->getNetworkInterfaces(&written);
+		if (networkInterfaceAPI->error) {
+			showError(networkInterfaceAPI->error);
+		} else {
+			uint64_t traffic = 0;
+			for (int i = 0; i < written; ++i) {
+				if (interfaces[i].received_bytes + interfaces[i].sent_bytes > traffic) {
+					interfaceName = interfaces[i].name;
+					traffic = interfaces[i].received_bytes + interfaces[i].sent_bytes;
+				}
+			}
+		}
+	}
+	int i = gui.comboBoxNetworkInterface->findText(interfaceName);
+	gui.comboBoxNetworkInterface->setCurrentIndex(i == -1 ? 1 : i);
 }
 
 void MainWindow::radioButtonPlayAlarmToggled(bool checked) {
@@ -404,7 +377,7 @@ void MainWindow::radioButtonPlayAlarmToggled(bool checked) {
 	QString savedFileName = groupPlayAlarm.readEntry("File");
 	int savedIndex = -1;
 	// Populating the combobox
-	for (int i = 0; i < availableSounds->size(); i++) {
+	for (int i = 0; i < availableSounds->size(); ++i) {
 		if (savedIndex == -1 && savedFileName.compare(availableSounds->at(i)) == 0)
 			savedIndex = i;
 		QFileInfo fileInfo = QFileInfo(availableSounds->at(i));
@@ -459,16 +432,16 @@ void MainWindow::killProcessClicked() {
 
 void MainWindow::startStopClicked() {
 	if (condition == NULL)
-		startClicked();
+		startCondition();
 	else
-		stopClicked();
+		stopCondition();
 }
 
-void MainWindow::startClicked() {
+void MainWindow::startCondition() {
 	// validating actions
-	int actionCheckedId = actionsGroup.checkedId();
+	ActionId actionCheckedId = (ActionId) actionsGroup.checkedId();
 	switch (actionCheckedId) {
-	case RADIOBUTTONSLEEP:
+	case SLEEP:
 		if (!gui.radioButtonSleepStandBy->isChecked() && !gui.radioButtonSleepSuspend->isChecked()
 				&& !gui.radioButtonSleepHibernate->isChecked()) {
 			KMessageBox::sorry(this, i18nc(
@@ -477,132 +450,115 @@ void MainWindow::startClicked() {
 			return;
 		}
 		break;
-	case RADIOBUTTONEXECUTECOMMAND:
+	case EXECUTECOMMAND:
 		if (gui.texteditExecuteCommand->toPlainText().isEmpty()) {
 			KMessageBox::sorry(this, i18n("A command must be specified."));
 			return;
 		}
 		break;
-	case RADIOBUTTONKILLPROCESS:
+	case KILLPROCESS:
 		if (pidKillProcess == 0) {
 			KMessageBox::sorry(this, i18n("You must select a program to kill."));
 			return;
 		}
+		if (!processExists(pidKillProcess))
+			return;
+		break;
+	default:
+		break;
 	}
 
 	// validating conditions
-	int seconds = 0;
-	int conditionCheckedId = conditionsGroup.checkedId();
-	if (conditionCheckedId == RADIOBUTTONCPU || conditionCheckedId == RADIOBUTTONMEMORY
-			|| conditionCheckedId == RADIOBUTTONNETWORK) {
+	qint64 msecs = 0;
+	ConditionId conditionCheckedId = (ConditionId) conditionsGroup.checkedId();
+	if (conditionCheckedId == CPU_ID || conditionCheckedId == MEMORY || conditionCheckedId
+			== NETWORK) {
 		QTime time;
 		switch (conditionCheckedId) {
-		case RADIOBUTTONCPU:
+		case CPU_ID:
 			time = gui.timeEditCPU->time();
 			break;
-		case RADIOBUTTONMEMORY:
+		case MEMORY:
 			time = gui.timeEditMemory->time();
 			break;
-		case RADIOBUTTONNETWORK:
+		default:
 			time = gui.timeEditNetwork->time();
-			break;
 		}
 
-		seconds = time.minute() * 60 + time.second();
-		if (seconds == 0) {
+		msecs = (time.minute() * 60 + time.second()) * 1000;
+		if (msecs == 0) {
 			KMessageBox::sorry(this, i18nc("@info The user has chosen a wrong time: 00:00",
 					"The specified time must be greater than zero."));
 			return;
 		}
 	}
-	// TODO Check if the chosen processes to watch for still exists
 
 	switch (conditionCheckedId) {
-	case RADIOBUTTONCPU:
-		condition = new CPU(seconds, 2, gui.spinBoxCPU->value(), gui.comboBoxCPU->currentIndex()
+	case CPU_ID:
+		condition = new CPU(msecs, 2000, gui.spinBoxCPU->value(), gui.comboBoxCPU->currentIndex()
 				== 0);
 		break;
-	case RADIOBUTTONMEMORY:
-		condition = new Memory(seconds, 4, gui.spinBoxMemory->value() * 1048576,
+	case MEMORY:
+		condition = new Memory(msecs, 4000, gui.spinBoxMemory->value() * 1048576,
 				gui.comboBoxMemory->currentIndex() == 0);
 		break;
-	case RADIOBUTTONNETWORK:
-		condition = new Network(seconds, 2, gui.comboBoxNetworkInterface->currentText(),
-				gui.spinBoxNetwork->value() * 1024, gui.comboBoxNetworkHigher->currentIndex() == 0,
+	case NETWORK:
+		if (networkInterfaceAPI == NULL)
+			networkInterfaceAPI = new NetworkInterface();
+		condition = new Network(networkInterfaceAPI, msecs, 2000,
+				gui.comboBoxNetworkInterface->currentText(), gui.spinBoxNetwork->value() * 1024,
+				gui.comboBoxNetworkHigher->currentIndex() == 0,
 				gui.comboBoxNetworkDirection->currentIndex() == 0 ? DOWNLOAD : UPLOAD);
 		break;
-	case RADIOBUTTONDATETIME:
-		seconds = QDateTime::currentDateTime().secsTo(gui.kdatetimewidgetDateTime->dateTime());
-		if (seconds <= 0) {
+	case DATETIME:
+		msecs = QDateTime::currentDateTime().msecsTo(gui.kdatetimewidgetDateTime->dateTime());
+		if (msecs <= 0) {
 			KMessageBox::sorry(this, i18nc("@info",
 					"You have selected a date in the past. Please input a future date."));
 			return;
 		}
-		condition = new DateTime(seconds);
+		condition = new DateTime(msecs);
 		break;
-	case RADIOBUTTONPROCESSDIES:
+	default:
 		if (pidProcessDies == 0) {
 			KMessageBox::sorry(this, i18nc("@info", "You must select a program to monitor."));
 			return;
 		}
-#ifdef SA_OPEN_PROCESS
-		if (sa_open_process() != 0) {
-			showError(ENOSYS);
+		if (!processExists(pidProcessDies))
 			return;
-		}
-#endif
-#if SA_VERSION_MAJOR > 0 || (SA_VERSION_MAJOR == 0 && SA_VERSION_SMALL >= 6)
-		struct sa_process_activity dst;
-		int ret = sa_get_process_activity(pidProcessDies, &dst);
-		if (ret == 0)
-			ret = sa_get_process_activity(pidKillProcess, &dst);
-#else
-		struct sa_process dst;
-		int ret = sa_get_process(pidProcessDies, &dst);
-		if (ret == 0)
-			ret = sa_get_process(pidKillProcess, &dst);
-#endif
-#ifdef SA_CLOSE_PROCESS
-		if (sa_close_process() != 0) {
-			showError(ENOSYS);
-			return;
-		}
-#endif
-		if (ret != 0) {
-			showError(ret);
-			return;
-		}
-		condition = new ProcessDies(2, 2, pidProcessDies);
+		condition = new ProcessDies(processAPI, 2000, 2000, pidProcessDies);
 	}
 
-	connect(condition, SIGNAL(finished()), this, SLOT(conditionFinished()));
-	condition->start(QThread::HighPriority);
+	connect(condition, SIGNAL(conditionAccomplished()), this, SLOT(conditionAccomplished()));
+	connect(condition, SIGNAL(conditionFailed(int)), this, SLOT(conditionFailed(int)));
 	gui.groupBoxConditions->setEnabled(false);
 	gui.groupBoxActions->setEnabled(false);
 	gui.pushButtonStartStop->setText(i18nc("@action:button Stop the running task", "Stop"));
 	gui.pushButtonStartStop->setIcon(
 			QIcon(iconLoader->loadIcon("process-stop", KIconLoader::Small)));
-	saveConfig(conditionCheckedId, actionCheckedId);
+	trayIcon.setStatus(KStatusNotifierItem::Active);
+	saveConfig(conditionCheckedId, actionCheckedId, msecs);
 }
 
-inline void MainWindow::saveConfig(int conditionCheckedId, int actionCheckedId) {
+inline void MainWindow::saveConfig(ConditionId conditionCheckedId, ActionId actionCheckedId,
+		qint64 msecs) {
 	KConfigGroup group = config->group("Conditions");
-	group.writeEntry("LastExecuted", conditionCheckedId);
+	group.writeEntry("LastExecuted", (int) conditionCheckedId);
 	KConfigGroup configGroup;
 	switch (conditionCheckedId) {
-	case RADIOBUTTONCPU:
+	case CPU_ID:
 		configGroup = config->group("ConditionCPU");
 		configGroup.writeEntry("Higher", gui.comboBoxCPU->currentIndex());
 		configGroup.writeEntry("Percentage", gui.spinBoxCPU->value());
 		configGroup.writeEntry("Time", gui.timeEditCPU->time().toString("mm:ss"));
 		break;
-	case RADIOBUTTONMEMORY:
+	case MEMORY:
 		configGroup = config->group("ConditionMemory");
 		configGroup.writeEntry("Higher", gui.comboBoxMemory->currentIndex());
 		configGroup.writeEntry("Amount", gui.spinBoxMemory->value());
 		configGroup.writeEntry("Time", gui.timeEditMemory->time().toString("mm:ss"));
 		break;
-	case RADIOBUTTONNETWORK:
+	case NETWORK:
 		configGroup = config->group("ConditionNetwork");
 		configGroup.writeEntry("Direction", gui.comboBoxNetworkDirection->currentIndex());
 		configGroup.writeEntry("Higher", gui.comboBoxNetworkHigher->currentIndex());
@@ -610,20 +566,23 @@ inline void MainWindow::saveConfig(int conditionCheckedId, int actionCheckedId)
 		configGroup.writeEntry("Interface", gui.comboBoxNetworkInterface->currentText());
 		configGroup.writeEntry("Time", gui.timeEditNetwork->time().toString("mm:ss"));
 		break;
-	case RADIOBUTTONDATETIME:
+	case DATETIME:
 		configGroup = config->group("ConditionDateTime");
-		configGroup.writeEntry("DateTime", gui.kdatetimewidgetDateTime->dateTime());
+		configGroup.writeEntry("Interval", msecs / 1000);
+		break;
+	default:
 		break;
 	}
 
 	group = config->group("Actions");
-	group.writeEntry("LastExecuted", gui.radioButtonShutdown->isChecked() ? 0 : actionCheckedId + 1);
+	group.writeEntry("LastExecuted", (int) actionCheckedId);
 	switch (actionCheckedId) {
-	case RADIOBUTTONSHUTDOWN:
+	case SHUTDOWN:
+	case REBOOT:
 		configGroup = config->group("ActionShutdown");
 		configGroup.writeEntry("Force", gui.checkBoxShutdownForce->isChecked());
 		break;
-	case RADIOBUTTONSLEEP:
+	case SLEEP:
 		Solid::PowerManagement::SleepState state;
 		configGroup = config->group("ActionSleep");
 		if (gui.radioButtonSleepStandBy->isChecked())
@@ -634,97 +593,98 @@ inline void MainWindow::saveConfig(int conditionCheckedId, int actionCheckedId)
 			state = Solid::PowerManagement::HibernateState;
 		configGroup.writeEntry("Mode", (int) state);
 		break;
-	case RADIOBUTTONEXECUTECOMMAND:
+	case EXECUTECOMMAND:
 		configGroup = config->group("ActionExecuteCommand");
 		configGroup.writeEntry("Command", gui.texteditExecuteCommand->toPlainText());
 		break;
-	case RADIOBUTTONPLAYALARM:
+	case PLAYALARM:
 		configGroup = config->group("ActionPlayAlarm");
 		configGroup.writeEntry("File", availableSounds->at(gui.comboBoxPlayAlarm->currentIndex()));
+		break;
+	default:
+		break;
 	}
 	config->sync();
 }
 
-void MainWindow::stopClicked() {
-	disconnect(condition, SIGNAL(finished()), this, SLOT(conditionFinished()));
-	connect(condition, SIGNAL(finished()), condition, SLOT(deleteLater()));
-	condition->aborted = 1;
-	condition = NULL;
+void MainWindow::conditionAccomplished() {
+#define PASSIVE_POPUP_TITLE i18nc("@info The name of the app in the title of a passive popup", "Sentinella")
+	stopCondition();
 
-	gui.groupBoxConditions->setEnabled(true);
-	gui.groupBoxActions->setEnabled(true);
-	gui.pushButtonStartStop->setText(i18nc("@action:button Verb in imperative form", "Start"));
-	gui.pushButtonStartStop->setIcon(QIcon(iconLoader->loadIcon("arrow-right", KIconLoader::Small)));
+	ActionId actionCheckedId = (ActionId) actionsGroup.checkedId();
+	switch (actionCheckedId) {
+	case SHUTDOWN:
+	case REBOOT:
+		action = new Shutdown(isKDE, gui.radioButtonShutdown->isChecked(),
+				gui.checkBoxShutdownForce->isChecked());
+		if (gui.radioButtonShutdown->isChecked())
+			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
+					"Condition met. Halting the system."), this);
+		else
+			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
+					"Condition met. Rebooting the system."), this);
+		break;
+	case SLEEP:
+		Solid::PowerManagement::SleepState state;
+		if (gui.radioButtonSleepStandBy->isChecked()) {
+			state = Solid::PowerManagement::StandbyState;
+			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
+					"Condition met. Setting the system's state to standby."), this);
+		} else if (gui.radioButtonSleepSuspend->isChecked()) {
+			state = Solid::PowerManagement::SuspendState;
+			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
+					"Condition met. Suspending the system."), this);
+		} else {
+			state = Solid::PowerManagement::HibernateState;
+			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
+					"Condition met. Hibernating the system."), this);
+		}
+		action = new Sleep(isKDE, state);
+		break;
+	case EXECUTECOMMAND:
+		action = new ExecuteCommand(gui.texteditExecuteCommand->toPlainText());
+		KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
+				"Condition met. Executing command."), this);
+		break;
+	case PLAYALARM:
+		action = new PlayAlarm(availableSounds->at(gui.comboBoxPlayAlarm->currentIndex()));
+		KPassivePopup::message(PASSIVE_POPUP_TITLE,
+				i18nc("@info", "Condition met. Throwing alarm."), this);
+		break;
+	default:
+		action = new KillProcess(pidKillProcess);
+		KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
+				"Condition met. Killing the selected program."), this);
+	}
+	action->execute();
+
+	if (actionCheckedId == PLAYALARM) {
+		trayIcon.setStatus(KStatusNotifierItem::NeedsAttention);
+		KMessageBox::information(this, i18n("Close this message to turn off the alarm."));
+		((PlayAlarm*) action)->stop();
+		trayIcon.setStatus(KStatusNotifierItem::Passive);
+		delete action;
+		action = NULL;
+	}
 }
 
-void MainWindow::conditionFinished() {
-#define PASSIVE_POPUP_TITLE i18nc("@info The name of the app in the title of a passive popup", "Sentinella")
-	int error = condition->error;
-	delete condition;
+void MainWindow::conditionFailed(int error) {
+	showError(error);
+	stopCondition();
+}
+
+void MainWindow::stopCondition() {
+	condition->stop();
+	condition->deleteLater();
+	disconnect(condition, SIGNAL(conditionAccomplished()), this, SLOT(conditionAccomplished()));
+	disconnect(condition, SIGNAL(conditionFailed(int)), this, SLOT(conditionFailed(int)));
 	condition = NULL;
 
-	if (error != 0) {
-		showError(error);
-	} else {
-		int actionCheckedId = actionsGroup.checkedId();
-		switch (actionCheckedId) {
-		case RADIOBUTTONSHUTDOWN:
-			action = new Shutdown(isKDE, gui.radioButtonShutdown->isChecked(),
-					gui.checkBoxShutdownForce->isChecked());
-			if (gui.radioButtonShutdown->isChecked())
-				KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
-						"Condition met. Halting the system."), this);
-			else
-				KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
-						"Condition met. Rebooting the system."), this);
-			break;
-		case RADIOBUTTONSLEEP:
-			Solid::PowerManagement::SleepState state;
-			if (gui.radioButtonSleepStandBy->isChecked()) {
-				state = Solid::PowerManagement::StandbyState;
-				KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
-						"Condition met. Setting the system's state to standby."), this);
-			} else if (gui.radioButtonSleepSuspend->isChecked()) {
-				state = Solid::PowerManagement::SuspendState;
-				KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
-						"Condition met. Suspending the system."), this);
-			} else {
-				state = Solid::PowerManagement::HibernateState;
-				KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
-						"Condition met. Hibernating the system."), this);
-			}
-			action = new Sleep(isKDE, state);
-			break;
-		case RADIOBUTTONEXECUTECOMMAND:
-			action = new ExecuteCommand(gui.texteditExecuteCommand->toPlainText());
-			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
-					"Condition met. Executing command."), this);
-			break;
-		case RADIOBUTTONPLAYALARM:
-			action = new PlayAlarm(availableSounds->at(gui.comboBoxPlayAlarm->currentIndex()));
-			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
-					"Condition met. Throwing alarm."), this);
-			break;
-		case RADIOBUTTONKILLPROCESS:
-			action = new KillProcess(pidKillProcess);
-			KPassivePopup::message(PASSIVE_POPUP_TITLE, i18nc("@info",
-					"Condition met. Killing the selected program."), this);
-			break;
-		}
-		action->execute();
-
-		if (actionCheckedId == RADIOBUTTONPLAYALARM) {
-			KMessageBox::information(this, i18n("Close this message to turn off the alarm."));
-			((PlayAlarm*) action)->stop();
-			delete action;
-			action = NULL;
-		}
-	}
-
 	gui.groupBoxConditions->setEnabled(true);
 	gui.groupBoxActions->setEnabled(true);
 	gui.pushButtonStartStop->setText(i18nc("@action:button Verb in imperative form", "Start"));
 	gui.pushButtonStartStop->setIcon(QIcon(iconLoader->loadIcon("arrow-right", KIconLoader::Small)));
+	trayIcon.setStatus(KStatusNotifierItem::Passive);
 }
 
 void MainWindow::showError(const int error) {
@@ -750,18 +710,61 @@ void MainWindow::showError(const int error) {
 }
 
 void MainWindow::showAboutApp() {
-	KAboutApplicationDialog* aboutAppDialog = new KAboutApplicationDialog(&aboutData, this);
+	KAboutApplicationDialog* aboutAppDialog = new KAboutApplicationDialog(
+			KCmdLineArgs::aboutData(), this);
 	aboutAppDialog->exec();
 	delete aboutAppDialog;
 }
 
 void MainWindow::closeEvent(QCloseEvent *event) {
-	if (event->spontaneous()) {
-		setVisible(false);
-		event->ignore();
-	} else {
-		event->accept();
+	if (!event->spontaneous())
+		return;
+	setVisible(false);
+	event->ignore();
+}
+
+void MainWindow::saveProperties(KConfigGroup& config) {
+	config.writeEntry("Visible", isVisible());
+}
+
+void MainWindow::readProperties(const KConfigGroup& config) {
+	if (config.readEntry("Visible", true))
+		show();
+}
+
+bool MainWindow::createProcess() {
+	processAPI = new Process();
+	if (processAPI->error != 0) {
+		showError(processAPI->error);
+		delete processAPI;
+		processAPI = NULL;
+		return false;
 	}
+	return true;
+}
+
+bool MainWindow::processExists(pid_t pid) {
+	if (processAPI == NULL || !createProcess())
+		return false;
+	if (processAPI->processExists(pid))
+		return true;
+	showError(processAPI->error);
+	return false;
+}
+
+bool MainWindow::getProcess(struct sa_process* procInfo) {
+	if (processAPI == NULL || !createProcess())
+		return false;
+	processAPI->getProcess(procInfo);
+	if (processAPI->error) {
+		showError(processAPI->error);
+		return false;
+	}
+	return true;
+}
+
+void MainWindow::actionButtonClicked(int id) {
+	gui.stackedWidgetActions->setCurrentIndex(id == 0 ? 0 : id - 1);
 }
 
 #include "MainWindow.moc"
